#=====================================================================
# Model CLSPL
#=====================================================================

#=====================================================================
param Model symbolic := "CLSPL";
param Dane symbolic;
param Wyniki symbolic := Model&"_"&Dane&".csv";
param Podsumowanie symbolic := "Podsumowanie.csv";

#=====================================================================
param L_tygodni > 0, integer;
set Tygodnie := 1..L_tygodni;

param L_dni_na_tydzien > 0, integer;

param L_produktow > 0, integer;
set Produkty := 1..L_produktow;

#=====================================================================
param Czas_pracy_na_tydzien > 0;

param Zapas_pocz {Produkty} >= 0;
param Gotowosc_pocz {Produkty} binary;
check sum {j in Produkty} Gotowosc_pocz[j] = 1;

param Czas_wykonywania {Produkty} >= 0;
param Czas_przezbr {Produkty} >= 0;
param Koszt_przezbr {Produkty} >= 0;
param Jedn_koszt_zapasow {Produkty} >= 0;

param Popyt_w_tygodniu {Produkty, Tygodnie} >= 0;

#=====================================================================
var Gotowosc {Produkty, Tygodnie union {0}} binary;
var Produkcja {Produkty, Tygodnie} >= 0;
var Zapas {Produkty, Tygodnie union {0}} >= 0;

var Calk_koszt_zapasow >= 0;
var Calk_koszt_przezbr >= 0;
var Calk_L_przezbr >= 0;

var Uruchomienie {Produkty, Tygodnie} binary;
var Przeniesienie {Produkty, Tygodnie union {0}} binary;
var Wylacznosc {Tygodnie} binary;

#=====================================================================
minimize FC_2_2_a_Koszty_calk:
	Calk_koszt_przezbr + Calk_koszt_zapasow;

subject to

	FC_2_2_a_Calk_koszt_przezbr:
		Calk_koszt_przezbr =
			sum {j in Produkty, t in Tygodnie}
				Koszt_przezbr[j] * Uruchomienie[j, t];

	FC_2_2_a_Calk_koszt_zapasow:
		Calk_koszt_zapasow =
			sum {j in Produkty, t in Tygodnie}
				Jedn_koszt_zapasow[j] * Zapas[j, t];

	Ogr_Calk_L_przezbr:
		Calk_L_przezbr =
			sum {j in Produkty, t in Tygodnie}
				Uruchomienie[j, t];

	#--------------------------------------------------------------------------------------------------
	Ogr_Zapas_pocz {j in Produkty}:
		Zapas[j, 0] = Zapas_pocz[j];

	Ogr_Gotowosc_pocz {j in Produkty}:
		Gotowosc[j, 0] = Gotowosc_pocz[j];

	#--------------------------------------------------------------------------------------------------
	Ogr_2_2_b_Bilans_zapasow {j in Produkty, t in Tygodnie}:
		Zapas[j, t-1] + Produkcja[j, t]
		= Popyt_w_tygodniu[j, t] + Zapas[j, t];

	Ogr_2_2_c_Produkcja_gdy_Gotowosc {j in Produkty, t in Tygodnie}:
		Czas_wykonywania[j] * Produkcja[j, t] + Czas_przezbr[j] * Uruchomienie[j, t]
			<= Czas_pracy_na_tydzien * Gotowosc[j, t];

	Ogr_2_2_d_Obciazenie_nie_wieksze_od_czasu_pracy {t in Tygodnie}:
		sum {j in Produkty}
			(Czas_wykonywania[j] *Produkcja[j, t] + Czas_przezbr[j] * Uruchomienie[j, t])
		<= Czas_pracy_na_tydzien;

	#--------------------------------------------------------------------------------------------------
	Ogr_Przeniesienie_pocz {j in Produkty}:
		Przeniesienie[j, 0] = 0;

	Ogr_2_2_e_Max_jedno_przeniesienie {t in Tygodnie}:
		sum {j in Produkty} Przeniesienie[j, t] <= 1;

	Ogr_2_2_f_Jezeli_Przeniesienie_to_Gotowosc {j in Produkty, t in Tygodnie}:
		Przeniesienie[j, t] <= Gotowosc[j, t-1];

	Ogr_2_2_g_Jezeli_Uruchomienie_lub_Przeniesienie_to_Gotowosc
		{j in Produkty, t in Tygodnie}:
			Uruchomienie[j, t] +Przeniesienie[j, t] = Gotowosc[j, t];

	Ogr_2_2_h_Uruchomienie_albo_Wylacznosc {j in Produkty, t in Tygodnie}:
		Uruchomienie[j, t] + Wylacznosc[t] <= 1;

	Ogr_2_2_i_Jezeli_dwa_Przeniesienia_to_Wylacznosc
		{j in Produkty, t in Tygodnie: t >= 2}:
			Przeniesienie[j, t] + Przeniesienie[j, t-1] <= 1 + Wylacznosc[t-1];

#=====================================================================

	solve;

	printf "Zmienna;Produkt;Tydzie;Warto" > Wyniki;

	printf "\n%s;;;%s", 'Model',Model >> Wyniki;
	printf "\n%s;;;%s", 'Dane',Dane >> Wyniki;
	printf "\n%s;;;%s", 'Data', time2str(gmtime(),'%F') >> Wyniki;
	printf "\n%s;;;%s", 'Godzina', time2str(gmtime(),'%H:%M') >> Wyniki;
	printf "\n%s;;;%d", 'L_tygodni', L_tygodni >> Wyniki;
	printf "\n%s;;;%d", 'L_produktow', L_produktow >> Wyniki;
	printf "\n%s;;;%d", 'Czas_pracy', Czas_pracy_na_tydzien >> Wyniki;

	printf "\n%s;;;%3.1f", 'Calk_koszt_zapasow', Calk_koszt_zapasow >> Wyniki;
	printf "\n%s;;;%3.1f", 'Calk_koszt_przezbr', Calk_koszt_przezbr >> Wyniki;
	printf "\n%s;;;%3.1f", 'Calk_L_przezbr', Calk_L_przezbr >> Wyniki;

	for {j in Produkty} {
		printf "\n%s;%d;;%d", 'Zapas_pocz', j, Zapas_pocz[j] >> Wyniki;
		printf "\n%s;%d;;%3.1f", 'Czas_wykonywania', j, Czas_wykonywania[j] >> Wyniki;
		printf "\n%s;%d;;%3.1f", 'Koszt_przezbr', j, Koszt_przezbr[j] >> Wyniki;
		printf "\n%s;%d;;%3.1f", 'Czas_przezbr', j, Czas_przezbr[j] >> Wyniki;
		printf "\n%s;%d;;%3.1f", 'Jedn_koszt_zapasow', j, Jedn_koszt_zapasow[j] >> Wyniki;
	}

	for {j in Produkty, t in Tygodnie} {
		printf "\n%s;%d;%d;%3.1f", 'Popyt', j, t, Popyt_w_tygodniu[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%3.1f", 'Produkcja', j, t, Produkcja[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%d", 'Uruchomienie', j, t, Uruchomienie[j, t] >> Wyniki;
	}

	for {j in Produkty, t in Tygodnie union {0}} {
		printf "\n%s;%d;%d;%d", 'Gotowosc', j, t, Gotowosc[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%3.1f", 'Zapas', j, t, Zapas[j, t] >> Wyniki;
		printf "\n%s;%d;%d;%d", 'Przeniesienie', j, t, Przeniesienie[j, t] >> Wyniki;
	}

	for {t in Tygodnie}
		printf "\n%s;;%d;%d", 'Wylacznosc', t, Wylacznosc[t] >> Wyniki;

	#--------------------------------------------------------------------------------------------------
	printf "%s;%s;%3.1f;%3.1f;%3.1f;%3.1f\n", Model, Dane,
		Calk_koszt_zapasow , Calk_koszt_przezbr, Calk_L_przezbr
		>> Podsumowanie ;

#=====================================================================
end;
